package neuroet;

import java.util.Arrays;
import java.util.Comparator;
/*A bunch of utilities for dealing with arrays, including simple matrix multiplication and stat functions*/

public final class ArrayUtilities
{
	static public double[] AverageDoubleArrays(double[][] toAverage)
	{

		double[] answer = new double[toAverage[0].length];
		double sum;

		for (int y = 0; y < answer.length;y++)
		{
			sum = 0.0;

			for (int x = 0; x < toAverage.length; x++)
			{
				sum += toAverage[x][y];
			}

			answer[y] = sum / toAverage.length;
		}

		return answer;
	}


	static public double[] SubtractVectors(double[] original, double[] toSubtract)
	{
		if (original.length != toSubtract.length)
			throw new IllegalArgumentException ("Can't Subtract Unequal length vectors");

		double[] answer = new double[original.length];

		for (int x = 0; x < original.length; x++)
		{
			answer[x] = original[x] - toSubtract[x];
		}

		return answer;
	}

	static public double[] AddVectors(double[] one, double[] two)
	{

		if (one.length != two.length)
			throw new IllegalArgumentException ("Can't add unequal length vectors");

		double[] answer = new double[one.length];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = one[x] + two[x];
		}

		return answer;
	}

	static public double AbsoluteAverage (double[] toAnalyze)  //returns the average of a double array, taking the absolute value
	{
		double answer = 0.0;

		for (int x = 0; x < toAnalyze.length; x++)
		{
			answer += Math.abs(toAnalyze[x]);
		}
		answer /= toAnalyze.length;

		return answer;
	}

        static public double SumOfSquaresError(double[] guess, double[] answ)
        {
          if (answ.length != guess.length)
            throw new IllegalArgumentException("Answer and guess length are not equal: " + answ.length + "\t" + guess.length);

          double answer = 0.0;

          for (int x = 0; x < answ.length; x++)
          {
            double err = answ[x] - guess[x];
            answer += err * err;
          }

          return answer;
        }

	static public double SumOfSquares(double[] error)
	{
		double answer = 0.0;

		for (int x = 0; x < error.length; x++)
		{
			answer += error[x] * error[x];
		}



		return answer;
	}

	//works, but private so its not called in NN

	static private double MeanSumOfSquares(double[] error)
	{
		return (SumOfSquares(error) / error.length);

	}

	static public double CalculateError(double[] guesses, double[] answers)//average percent error
	{
		double answer = 0.0;

		if (guesses.length != answers.length)
			throw new IllegalArgumentException("bad lengths in Calculate Error");


		for (int x = 0; x < guesses.length; x++)
		{
			answer += Math.abs((guesses[x] - answers[x])/answers[x]);
		}

		return answer / guesses.length;

	}


        public static double GetXthPercentile(double[] ar, double per)
        {
          if (per < 0.0 || per > 1.0)
            throw new IllegalArgumentException("Illegal Percent: " + per);

          if (ar.length == 0)
            throw new IllegalArgumentException("Array must have a length > 0");

          double[] cop = (double[])ar.clone();

          Arrays.sort(cop);

          int ind = cop.length - 1;
          ind *= per;

          return cop[ind];

        }

        public static Object GetXthPercentile (Object[] ar, Comparator c, double per)
        {
          if (per < 0.0 || per > 1.0)
            throw new IllegalArgumentException("Illegal Percent: " + per);

          if (ar.length == 0)
            throw new IllegalArgumentException("Array must have a length > 0");
          Arrays.sort(ar, c);
          int ind = ar.length - 1;
          ind *= per;

          return ar[ind];


        }

        public static double[] CreateUnitVector(double[] ar)
        {
          double len = ComputeVectorLength(ar);

          return MultiplyArray(1.0 / len, ar);

        }

        public static double ComputeVectorLength(double[] ar)
        {
          double answer = 0;

          for (int x = 0; x < ar.length; x++)
          {
            double curr = ar[x];
            answer += curr * curr;
          }

          return Math.sqrt(answer);

        }

	public static double[] MultiplyArray(double scalar, double[] ar)
	{
		double[] answer = new double[ar.length];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = ar[x] * scalar;
		}

		return answer;
	}

	static public double MultiplyVectors(double[] one, double[] two)
	{

		if (one.length != two.length)
			throw new IllegalArgumentException("Can't multiply these vectors (Bad Length): "+ one.length + "\t" + two.length);

		double answer = 0.0;

		for (int x = 0; x < one.length; x++)
		{
			answer  += one[x] * two[x];
		}

		return answer;

	}

	static public void NegateArray(double[] array) //negates array in place
	{
		for (int x = 0; x < array.length; x++)
		{
			array[x] = -array[x];
		}
	}

	static public double[] IncrementArray(double[] array, double increment)//can also decrement
	{
		double[] answer = new double[array.length];

		for (int x = 0; x < answer.length; x++)
		{
			answer[x] = array[x] + increment;
		}

		return answer;
	}

	static public double CalculateMean (double[] ar)
	{
		double answer = 0.0;

		for (int x = 0; x < ar.length; x++)
		{
			answer += ar[x];
		}

		return answer / ar.length;
	}


	static public double CalculateStdDev(double[] ar)
	{
		double mean = CalculateMean(ar);

		double sum = 0.0;
		double temp;

		for (int x = 0; x < ar.length; x++)
		{
			temp = ar[x] - mean;
			sum += temp * temp;
		}

		return Math.sqrt(sum/ (ar.length - 1));

	}

      static public double CalculateSSYY(double[] ar)
      {
        final double mean = CalculateMean(ar);
        double answer = 0.0;

        for (int x = 0; x < ar.length; x++)
        {
          double tr = ar[x] - mean;

          answer += tr * tr;
        }

        return answer;

      }

      static public double CalculateRSquared(double[] guesses, double[] answers)
      {
    	if (guesses.length != answers.length)
    		throw new IllegalArgumentException("Unequal array lengths");

    	double sse = 0.0;  //sum squares error
    	double error;
    	double tot;
    	double ssyy = 0.0;  //sum of squares regression + error

    	double meanY = CalculateMean(answers);

    	for (int x = 0; x < guesses.length; x++)
    	{
    		error = guesses[x] - answers[x];
    		sse += error * error;
    		tot = answers[x] - meanY;
    		ssyy += tot * tot;
    	}


    	return 1.0 - sse / ssyy;

    }

    static public double SimplePow(double v, int p)
    {

      double answer = 1.0;

      if (p > 0)
      {
        while (p > 0)
        {
          answer *= v;
          p--;
        }
      }
      else if (p < 0)
      {
        while(p < 0)
        {
         answer /= v;
         p++;
        }
      }

      return answer;

    }

    static public double CalculateCentralMoment(int order, double[] sample)
    {
      final double mean = CalculateMean(sample);

      final double dOrder = (double)order;
      double total = 0.0;

      for (int x = 0; x < sample.length; x++)
      {
        total += /*ArrayUtilities.SimplePow(sample[x] - mean, order);*/Math.pow(sample[x] - mean, dOrder);
      }

      return total / sample.length;


    }



    /*The skew and kurtosis measure how the values are distributed around the mode (see below).
    A skew value of zero indicates that the values are evenly distributed on both sides of the mode.
    A negative skew indicates an uneven distribution with a higher than normal distribution of values
    to the right of the mode, a positive value for the skew indicates a larger than normal distribution
    of values to the left of the mode. The kurtosis of the distribution indicates how narrow or broad the
    distribution is. A positive value for the kurtosis indicates a narrower distribution than a normal Gaussian,
    a negative value indicates a flatter and broader distribution. A normal Gaussian distribution has a kurtosis
    of zero.*/

    static public double CalculateSkew(double[] sample)
    {
      return (CalculateCentralMoment(3, sample)) / (Math.pow(CalculateCentralMoment(2, sample),1.5));
    }

    static public double CalculateKurtosis(double[] sample)
    {
      return (CalculateCentralMoment(4, sample)) / (Math.pow(CalculateCentralMoment(2, sample),2.0)) - 3.0;
    }


    static public double FindMax (double[] ar)
    {
    	double answer = Double.NEGATIVE_INFINITY;

    	for (int x = 0; x < ar.length; x++)
    	{
    		if (ar[x] > answer)
    			answer = ar[x];
    	}
    	return answer;
    }

    static public double FindMin(double[] ar)
    {

    	double answer = Double.POSITIVE_INFINITY;

    	for (int x = 0; x < ar.length; x++)
    	{
    		if (ar[x] < answer)
    			answer = ar[x];
    	}

    	return answer;

    }

    static public double[][] RotateMatrix(double[][] mat)
    {

            double[][] toReturn = new double[mat[0].length][mat.length];


            for (int x = 0; x < toReturn.length; x++)
            {
                    for (int y = 0; y < mat.length; y++)
                    {
                            toReturn[x][y] = mat[y][x];
                    }
            }

            return toReturn;
    }

    static public double CalculateMedian(double[] toExamine)
    {

      if (toExamine.length < 1)
         throw new IllegalArgumentException("Need at least one element to calculate median");


      toExamine = (double[])toExamine.clone();  //don't modify original array
      java.util.Arrays.sort(toExamine);

      int middle = toExamine.length / 2;

      if (toExamine.length % 2 == 1)
         return toExamine[middle];
      else
        return (toExamine[middle - 1] + toExamine[middle]) / 2.0;

    }

    //Shouldn't be ran with arrays less than length 23
    public static boolean CalculateRunCorrelation(double[] one, double[] two)
    {
      if (one.length != two.length)
         throw new IllegalArgumentException("Invalid Array Length in CalculateRunCorrelation: " + one.length + "\t" + two.length);

      DoubleHolder[] tempArr = new DoubleHolder[one.length];
      double median = ArrayUtilities.CalculateMedian(two);

      int n1 = 0;
      int n2 = 0;

      int w = 0;
      for (int x = 0; x < tempArr.length; x++)
      {
        if (two[x] == median)
           continue;

        if (two[x] > median)
           n1++;
        else
           n2++;

        tempArr[w] = new DoubleHolder(one[x], two[x]);
        w++;
      }

      if (n1 < 11 || n2 < 11)
         throw new IllegalArgumentException("n1 or n2 < 11: " + n1 + "\t" + n2);

      DoubleHolder[] ar = new DoubleHolder[w];

      System.arraycopy(tempArr, 0, ar, 0, ar.length);

      Arrays.sort(ar, DoubleHolder.firstAscending);


      int runs = 1;
      double prev = ar[0].GetSecond();



      for (int x = 1; x < ar.length; x++)
      {
        double next = ar[x].GetSecond();

        if ((prev > median) != (next > median))
           runs++;

        prev = next;

      }



      double expected = ((2.0 * n1 * n2) / (n1 + n2)) + 1.0;
      double stdError = Math.sqrt(((2.0 * n1 * n2) * (2.0 * n1 * n2 - n1 - n2)) /
      ((n1 + n2) * (n1 + n2) * (n1 + n2 - 1)));

      double z = (runs - expected) / stdError;

      if (z < -2.575) //we have too few runs (.005 significance)
         return true;
      else
        return false;


    }

    public static double CalculateCDFBinomial(double p, int n, int x)
    {
      double q = 1.0 - p;
      double current = Math.pow(q, n);
      double total = current;

      if (x < 0)
         return 0.0;

      if (q <= 0.0)
      {
        if (x < n)
          return 0.0;
      }

      if (x >= n)
         return 1.0;

      for(int k = 1; k <= x; k++)
      {
        current = current * ((n - k + 1.0) / k) * (p / q);
        total += current;
      }

      return total;

    }

    public static double BinomialBackwardRecursive(int k, int n, double p, double q)
    {
      if (k == 0)
         return Math.pow(q, n);
      else
        return BinomialBackwardRecursive(k - 1, n, p, q) * ((n - k + 1.0) / k) * (p / q);
    }

    public static double BinomialForwardRecursive(int k, int n, double p, double q)
    {
       if (k == 0)
          return Math.pow(p, n);
       else
          return BinomialForwardRecursive(k - 1, n, p, q) * ((n - k + 1.0) / k) * (p / q);
    }

    public static double[] StripArrayOfValues(double[] myArray, double badVal)
    {
      double[] ar = new double[myArray.length];

      int y = 0;
      for (int x = 0; x < myArray.length; x++)
      {
        if (myArray[x] == badVal)
           continue;

        ar[y] = myArray[x];
        y++;
      }

      double[] answer = new double[y];

      System.arraycopy(ar, 0, answer, 0, y);

      return answer;

    }


}
